@@ -1,24 +1,7 @@
 /*
- * Copyright (C) 2015, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2015 - 2017, Stephan Mueller <smueller@chronox.de>
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, and the entire permission notice in its entirety,
- *    including the disclaimer of warranties.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote
- *    products derived from this software without specific prior
- *    written permission.
- *
- * ALTERNATIVELY, this product may be distributed under the terms of
- * the GNU General Public License, in which case the provisions of the GPL2
- * are required INSTEAD OF the above restrictions.  (This clause is
- * necessary due to a potential bad interaction between the GPL and
- * the restrictions contained in a BSD-style copyright.)
+ * License: see LICENSE file in root directory
  *
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
  * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
@@ -69,121 +52,40 @@
 #include <getopt.h>
 #include <errno.h>
 #include <ctype.h>
+#include <dlfcn.h>
+#include <libgen.h>
 
 #include <kcapi.h>
 
+#include "app-internal.h"
+
 static uint8_t fipscheck_hmackey[] = "orboDeJITITejsirpADONivirpUkvarP";
 static uint8_t hmaccalc_hmackey[] = "FIPS-FTW-RHT2009";
 
-static void usage(char *hashname)
+static void usage(char *name)
 {
-	fprintf(stderr, "\n%ssum - calculation of hash sum (Using Linux Kernel Crypto API)\n", hashname);
+	fprintf(stderr, "\n%s - calculation of hash sum (Using Linux Kernel Crypto API)\n", basename(name));
 	fprintf(stderr, "\nUsage:\n");
-	fprintf(stderr, "\t%ssum [OPTION] ... [FILE] ...\n", hashname);
+	fprintf(stderr, "\t%s [OPTION] ... [FILE] ...\n", basename(name));
 	fprintf(stderr, "Options:\n");
 	fprintf(stderr, "\t-c --check [FILE]\tVerify hash sums from file\n");
-	fprintf(stderr, "\t-q --quiet\t\tDo not print out verification result for every file\n");
+	fprintf(stderr, "\t-q --quiet\t\tDo not print out verification result for\n");
+	fprintf(stderr, "\t\t\t\tevery file\n");
 	fprintf(stderr, "\t-s --status\t\tResult of verification given with return code\n");
-	fprintf(stderr, "\t-k --hkey [HEX HMAC KEY]\tPerform HMAC verification with given key\n");
-	fprintf(stderr, "\t-b --bkey [HMAC KEY]\tPerform HMAC verification with given key\n");
+	fprintf(stderr, "\t-k --hkey [HEX KEY]\tPerform HMAC verification with given key\n");
+	fprintf(stderr, "\t-b --bkey [KEY]\t\tPerform HMAC verification with given key\n");
 	fprintf(stderr, "\t-h --help\t\tPrint this help text\n");
 	fprintf(stderr, "\t-v --version\t\tShow version\n");
 }
 
-static void version(char *hashname)
+static void version(char *name)
 {
 	char version[20];
 
 	memset(version, 0, 20);
 	kcapi_versionstring(version, 20);
 	
-	fprintf(stderr, "%ssum: %s\n", hashname, version);
-}
-
-static char hex_char_map_l[] = { '0', '1', '2', '3', '4', '5', '6', '7',
-				 '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
-static char hex_char_map_u[] = { '0', '1', '2', '3', '4', '5', '6', '7',
-				 '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
-static char hex_char(uint32_t bin, int u)
-{
-	if (bin < sizeof(hex_char_map_l))
-		return (u) ? hex_char_map_u[bin] : hex_char_map_l[bin];
-	return 'X';
-}
-
-static void bin2hex(const uint8_t *bin, uint32_t binlen,
-		    char *hex, uint32_t hexlen, int u)
-{
-	uint32_t i = 0;
-	uint32_t chars = (binlen > (hexlen / 2)) ? (hexlen / 2) : binlen;
-
-	for (i = 0; i < chars; i++) {
-		hex[(i*2)] = hex_char((bin[i] >> 4), u);
-		hex[((i*2)+1)] = hex_char((bin[i] & 0x0f), u);
-	}
-}
-
-static int bin_char(uint8_t hex)
-{
-	if (48 <= hex && 57 >= hex)
-		return (hex - 48);
-	if (65 <= hex && 70 >= hex)
-		return (hex - 55);
-	if (97 <= hex && 102 >= hex)
-		return (hex - 87);
-	return 0;
-}
-
-static void hex2bin(const char *hex, uint32_t hexlen,
-		    uint8_t *bin, uint32_t binlen)
-{
-	uint32_t i = 0;
-	uint32_t chars = (binlen > (hexlen / 2)) ? (hexlen / 2) : binlen;
-
-	for (i = 0; i < chars; i++) {
-		bin[i] = bin_char(hex[(i*2)]) << 4;
-		bin[i] |= bin_char(hex[((i*2)+1)]);
-	}
-}
-
-static int hex2bin_alloc(const char *hex, uint32_t hexlen,
-			 uint8_t **bin, uint32_t *binlen)
-{
-	uint8_t *out = NULL;
-	uint32_t outlen = 0;
-
-	if (!hexlen)
-		return -EINVAL;
-
-	outlen = (hexlen + 1) / 2;
-
-	out = calloc(1, outlen);
-	if (!out)
-		return -errno;
-
-	hex2bin(hex, hexlen, out, outlen);
-	*bin = out;
-	*binlen = outlen;
-	return 0;
-}
-
-static void bin2print(const uint8_t *bin, uint32_t binlen,
-		      const char *filename, FILE *outfile)
-{
-	char *hex;
-	uint32_t hexlen = binlen * 2 + 1;
-
-	hex = calloc(1, hexlen);
-	if (!hex)
-		return;
-	bin2hex(bin, binlen, hex, hexlen - 1 , 0);
-	/* fipshmac does not want the file name :-( */
-	if (outfile != stdout) {
-		fprintf(outfile, "%s\n", hex);
-	} else {
-		fprintf(outfile, "%s  %s\n", hex, filename);
-	}
-	free(hex);
+	fprintf(stderr, "%s: %s\n", basename(name), version);
 }
 
 static int hasher(struct kcapi_handle *handle, char *filename,
@@ -239,7 +141,7 @@ static int hasher(struct kcapi_handle *handle, char *filename,
 				ret = 0;
 
 		} else {
-			bin2print(md, ret, filename, outfile);
+			bin2print(md, ret, filename, outfile, 1);
 			ret = 0;
 		}
 	} else {
@@ -356,6 +258,7 @@ static int process_checkfile(char *hashname, char *checkfile, char *targetfile,
 	FILE *file = NULL;
 	int ret = 0;
 	struct kcapi_handle *handle;
+
 	/*
 	 * A file can have up to 4096 characters, so a complete line has at most
 	 * 4096 bytes (file name) + 128 bytes (SHA512 hex value) + 2 spaces +
@@ -467,6 +370,8 @@ static int fipscheck_self(char *hash,
 #define BUFSIZE 4096
 	char selfname[BUFSIZE];
 	int32_t selfnamesize = 0;
+	Dl_info info;
+	void *dl, *sym;
 
 #ifdef HAVE_SECURE_GETENV
 	if (secure_getenv("KCAPI_HASHER_FORCE_FIPS")) {
@@ -502,6 +407,7 @@ static int fipscheck_self(char *hash,
 		goto out;
 	}
 
+	/* Integrity check of our application. */
 	memset(selfname, 0, sizeof(selfname));
 	selfnamesize = readlink("/proc/self/exe", selfname, BUFSIZE);
 	if (selfnamesize >= BUFSIZE || selfnamesize < 0) {
@@ -510,10 +416,46 @@ static int fipscheck_self(char *hash,
 		goto out;
 	}
 
-	ret = -ENOMEM;
 	checkfile = get_hmac_file(selfname);
-	if (!checkfile)
+	if (!checkfile) {
+		ret = -ENOMEM;
 		goto out;
+	}
+
+	ret = process_checkfile(hash, checkfile, selfname, CHK_STATUS,
+				hmackey, hmackeylen);
+	if (ret)
+		goto out;
+
+	/* Integrity check of shared libkcapi.so file. */
+	memset(selfname, 0, sizeof(selfname));
+	snprintf(selfname, (sizeof(selfname) - 1), "libkcapi.so.%u",
+		 KCAPI_MAJVERSION);
+	dl = dlopen(selfname, RTLD_NODELETE|RTLD_NOLOAD|RTLD_LAZY);
+	if (dl == NULL) {
+		fprintf(stderr, "dlopen of file %s failed\n", selfname);
+		ret = -EFAULT;
+		goto out;
+	}
+
+	memset(selfname, 0, sizeof(selfname));
+	sym = dlsym(dl, "kcapi_md_init");
+	if (sym == NULL || !dladdr(sym, &info)) {
+		fprintf(stderr, "finding symbol kcapi_md_init failed\n");
+		ret = -EFAULT;
+		goto out;
+	}
+
+	strncpy(selfname, info.dli_fname, (sizeof(selfname) - 1));
+
+	dlclose(dl);
+
+	free(checkfile);
+	checkfile = get_hmac_file(selfname);
+	if (!checkfile) {
+		ret = -ENOMEM;
+		goto out;
+	}
 
 	ret = process_checkfile(hash, checkfile, selfname, CHK_STATUS,
 				hmackey, hmackeylen);
@@ -559,7 +501,7 @@ int main(int argc, char *argv[])
 		{"version", 0, 0, 'v'},
 		{"hkey", 1, 0, 'k'},
 		{"bkey", 1, 0, 'b'},
-		{"help", 1, 0, 'h'},
+		{"help", 0, 0, 'h'},
 		{0, 0, 0, 0}
 	};
 
@@ -643,10 +585,82 @@ int main(int argc, char *argv[])
 		if (-1 == c)
 			break;
 		switch (c) {
+			case 0:
+				switch (opt_index) {
+				case 0:
+					if (checkfile)
+						free(checkfile);
+					checkfile = strdup(optarg);
+					if (!checkfile) {
+						fprintf(stderr, "Error copying file name: %s\n",
+							strerror(errno));
+						goto out;
+					}
+					break;
+				case 1:
+					loglevel = CHK_QUIET;
+					break;
+				case 2:
+					loglevel = CHK_STATUS;
+					break;
+				case 3:
+					version(argv[0]);
+					ret = 0;
+					goto out;
+					break;
+				case 4:
+					if (hmackey &&
+					    hmackey != fipscheck_hmackey &&
+					    hmackey != hmaccalc_hmackey) {
+						kcapi_memset_secure(hmackey, 0,
+								    hmackeylen);
+						free(hmackey);
+						hmackey = NULL;
+					}
+					if (hex2bin_alloc(optarg,
+							  strlen(optarg),
+							  &hmackey,
+							  &hmackeylen)) {
+						fprintf(stderr, "Cannot allocate memory for HMAC key\n");
+						goto out;
+					}
+					if (get_hmac_cipherstring(hash,
+							HASHNAMESIZE))
+						goto out;
+					break;
+				case 5:
+					if (hmackey &&
+					    hmackey != fipscheck_hmackey &&
+					    hmackey != hmaccalc_hmackey) {
+						kcapi_memset_secure(hmackey, 0,
+								    hmackeylen);
+						free(hmackey);
+						hmackey = NULL;
+					}
+					hmackey = (uint8_t *)strdup(optarg);
+					if (!hmackey) {
+						fprintf(stderr, "Cannot allocate memory for HMAC key\n");
+						goto out;
+					}
+					hmackeylen = strlen(optarg);
+					if (get_hmac_cipherstring(hash,
+								HASHNAMESIZE))
+						goto out;
+					break;
+				case 6:
+					usage(argv[0]);
+					ret = 0;
+					goto out;
+				}
+				break;
+
 			case 'v':
-				version(hash);
-				return 0;
+				version(argv[0]);
+				ret = 0;
+				goto out;
 			case 'c':
+				if (checkfile)
+					free(checkfile);
 				checkfile = strdup(optarg);
 				if (!checkfile) {
 					fprintf(stderr, "Error copying file name: %s\n",
@@ -661,10 +675,17 @@ int main(int argc, char *argv[])
 				loglevel = CHK_STATUS;
 				break;
 			case 'h':
-				usage(hash);
+				usage(argv[0]);
 				ret = 0;
 				goto out;
 			case 'k':
+				if (hmackey && hmackey != fipscheck_hmackey &&
+				    hmackey != hmaccalc_hmackey) {
+					kcapi_memset_secure(hmackey, 0,
+							    hmackeylen);
+					free(hmackey);
+					hmackey = NULL;
+				}
 				if (hex2bin_alloc(optarg, strlen(optarg),
 						  &hmackey, &hmackeylen)) {
 					fprintf(stderr, "Cannot allocate memory for HMAC key\n");
@@ -674,6 +695,13 @@ int main(int argc, char *argv[])
 					goto out;
 				break;
 			case 'b':
+				if (hmackey && hmackey != fipscheck_hmackey &&
+				    hmackey != hmaccalc_hmackey) {
+					kcapi_memset_secure(hmackey, 0,
+							    hmackeylen);
+					free(hmackey);
+					hmackey = NULL;
+				}
 				hmackey = (uint8_t *)strdup(optarg);
 				if (!hmackey) {
 					fprintf(stderr, "Cannot allocate memory for HMAC key\n");
@@ -684,7 +712,7 @@ int main(int argc, char *argv[])
 					goto out;
 				break;
 			default:
-				usage(hash);
+				usage(argv[0]);
 				goto out;
 		}
 	}
@@ -702,6 +730,8 @@ int main(int argc, char *argv[])
 		}
 
 		targetfile = argv[optind];
+		if (checkfile)
+			free(checkfile);
 		checkfile = get_hmac_file(targetfile);
 		if (!checkfile)
 			goto out;
