@@ -1,6 +1,6 @@
 /* Kernel crypto API AF_ALG SP800-108 / SP800-132 KDF API
  *
- * Copyright (C) 2016, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2016 - 2017, Stephan Mueller <smueller@chronox.de>
  *
  * License: see COPYING file in root directory
  *
@@ -78,14 +78,6 @@ static inline uint32_t _bswap32(uint32_t x)
 #error "Endianess not defined"
 #endif
 
-/* convert 32 bit integer into its string representation */
-static inline void kcapi_kdf_cpu_to_be32(uint32_t val, uint8_t *buf)
-{
-	uint32_t *a = (uint32_t *)buf;
-
-	*a = be_bswap32(val);
-}
-
 DSO_PUBLIC
 int32_t kcapi_kdf_dpi(struct kcapi_handle *handle,
 		      const uint8_t *src, uint32_t slen,
@@ -95,8 +87,7 @@ int32_t kcapi_kdf_dpi(struct kcapi_handle *handle,
 	int32_t err = 0;
 	uint8_t *dst_orig = dst;
 	uint8_t Ai[h];
-	uint32_t i = 1;
-	uint8_t iteration[sizeof(uint32_t)];
+	uint32_t i = be_bswap32(1);
 
 	if (dlen > INT_MAX)
 		return -EMSGSIZE;
@@ -122,8 +113,7 @@ int32_t kcapi_kdf_dpi(struct kcapi_handle *handle,
 		if (err < 0)
 			goto err;
 
-		kcapi_kdf_cpu_to_be32(i, iteration);
-		err = kcapi_md_update(handle, iteration, sizeof(uint32_t));
+		err = kcapi_md_update(handle, (uint8_t *)&i, sizeof(uint32_t));
 		if (err < 0)
 			goto err;
 		if (src && slen) {
@@ -147,7 +137,7 @@ int32_t kcapi_kdf_dpi(struct kcapi_handle *handle,
 				goto err;
 			dlen -= h;
 			dst += h;
-			i++;
+			i = i + be_bswap32(1);
 		}
 	}
 
@@ -170,8 +160,7 @@ int32_t kcapi_kdf_fb(struct kcapi_handle *handle,
 	uint8_t *dst_orig = dst;
 	const uint8_t *label;
 	uint32_t labellen = 0;
-	uint32_t i = 1;
-	uint8_t iteration[sizeof(uint32_t)];
+	uint32_t i = be_bswap32(1);
 
 	if (dlen > INT_MAX)
 		return -EMSGSIZE;
@@ -199,8 +188,7 @@ int32_t kcapi_kdf_fb(struct kcapi_handle *handle,
 		if (err)
 			goto err;
 
-		kcapi_kdf_cpu_to_be32(i, iteration);
-		err = kcapi_md_update(handle, iteration, sizeof(uint32_t));
+		err = kcapi_md_update(handle, (uint8_t *)&i, sizeof(uint32_t));
 		if (err)
 			goto err;
 		if (labellen) {
@@ -224,7 +212,7 @@ int32_t kcapi_kdf_fb(struct kcapi_handle *handle,
 				goto err;
 			dlen -= h;
 			dst += h;
-			i++;
+			i = i + be_bswap32(1);
 		}
 	}
 
@@ -243,8 +231,7 @@ int32_t kcapi_kdf_ctr(struct kcapi_handle *handle,
 	uint32_t h = kcapi_md_digestsize(handle);
 	int32_t err = 0;
 	uint8_t *dst_orig = dst;
-	uint32_t i = 1;
-	uint8_t iteration[sizeof(uint32_t)];
+	uint32_t i = be_bswap32(1);
 
 	if (dlen > INT_MAX)
 		return -EMSGSIZE;
@@ -253,8 +240,7 @@ int32_t kcapi_kdf_ctr(struct kcapi_handle *handle,
 		return -EFAULT;
 
 	while (dlen) {
-		kcapi_kdf_cpu_to_be32(i, iteration);
-		err = kcapi_md_update(handle, iteration, sizeof(uint32_t));
+		err = kcapi_md_update(handle, (uint8_t *)&i, sizeof(uint32_t));
 		if (err)
 			goto err;
 
@@ -280,7 +266,7 @@ int32_t kcapi_kdf_ctr(struct kcapi_handle *handle,
 
 			dlen -= h;
 			dst += h;
-			i++;
+			i = i + be_bswap32(1);
 		}
 	}
 
@@ -311,7 +297,7 @@ int32_t kcapi_hkdf(const char *hashname,
 	uint8_t ctr = 0x01;
 	struct kcapi_handle *handle = NULL;
 
-	if (!ikm || !ikmlen)
+	if (!ikm || !ikmlen || !dst)
 		return -EINVAL;
 
 	err = kcapi_md_init(&handle, hashname, 0);
@@ -326,7 +312,8 @@ int32_t kcapi_hkdf(const char *hashname,
 	}
 
 	if (h > HKDF_MAXHASH) {
-		kcapi_dolog(LOG_ERR, "Null salt size too small for hash\n");
+		kcapi_dolog(KCAPI_LOG_ERR,
+			    "Null salt size too small for hash\n");
 		err = -EFAULT;
 		goto err;
 	}
@@ -381,6 +368,12 @@ int32_t kcapi_hkdf(const char *hashname,
 			err = kcapi_md_final(handle, prk_tmp, h);
 			if (err < 0)
 				goto err;
+
+			/* Shut up Clang */
+			if (!dst) {
+				err = -EFAULT;
+				goto err;
+			}
 			memcpy(dst, prk_tmp, dlen);
 			dlen = 0;
 		} else {
@@ -536,12 +529,11 @@ int32_t kcapi_pbkdf(const char *hashname,
 		    uint8_t *key, uint32_t keylen)
 {
 	struct kcapi_handle *handle;
-	uint32_t h, i = 1;
+	uint32_t h, i = be_bswap32(1);
 #define MAX_DIGESTSIZE 64
 	uint8_t u[MAX_DIGESTSIZE] __attribute__ ((aligned (sizeof(uint64_t))));
 	uint8_t T[MAX_DIGESTSIZE] __attribute__ ((aligned (sizeof(uint64_t)))) =
 									{ 0 };
-	uint8_t iteration[sizeof(uint32_t)];
 	int32_t err = 0;
 
 	if (keylen > INT_MAX)
@@ -553,7 +545,8 @@ int32_t kcapi_pbkdf(const char *hashname,
 
 	h = kcapi_md_digestsize(handle);
 	if (h > sizeof(u)) {
-		kcapi_dolog(LOG_ERR, "Programming error in file %s at line %u\n",
+		kcapi_dolog(KCAPI_LOG_ERR,
+			    "Programming error in file %s at line %u\n",
 			    __FILE__, __LINE__);
 		err = -EFAULT;
 		goto err;
@@ -573,13 +566,11 @@ int32_t kcapi_pbkdf(const char *hashname,
 	while (keylen) {
 		uint32_t j;
 
-		kcapi_kdf_cpu_to_be32(i, iteration);
-
 		err = kcapi_md_update(handle, salt, saltlen);
 		if (err < 0)
 			goto err;
 
-		err = kcapi_md_update(handle, iteration, sizeof(uint32_t));
+		err = kcapi_md_update(handle, (uint8_t *)&i, sizeof(uint32_t));
 		if (err < 0)
 			goto err;
 
@@ -607,7 +598,7 @@ int32_t kcapi_pbkdf(const char *hashname,
 		} else {
 			keylen -= h;
 			key += h;
-			i++;
+			i = i + be_bswap32(1);
 		}
 	}
 
