@@ -1,6 +1,6 @@
 /* Kernel crypto API AF_ALG interface code
  *
- * Copyright (C) 2014 - 2016, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2015 - 2017, Stephan Mueller <smueller@chronox.de>
  *
  * License: see COPYING file in root directory
  *
@@ -48,7 +48,7 @@ void kcapi_memset_secure(void *s, int c, uint32_t n)
 /************************************************************
  * Logging logic
  ************************************************************/
-int kcapi_verbosity_level = LOG_ERR;
+int kcapi_verbosity_level = KCAPI_LOG_ERR;
 
 void kcapi_dolog(int severity, const char *fmt, ...)
 {
@@ -67,16 +67,16 @@ void kcapi_dolog(int severity, const char *fmt, ...)
 	va_end(args);
 
 	switch (severity) {
-	case LOG_DEBUG:
+	case KCAPI_LOG_DEBUG:
 		snprintf(sev, sizeof(sev), "Debug");
 		break;
-	case LOG_VERBOSE:
+	case KCAPI_LOG_VERBOSE:
 		snprintf(sev, sizeof(sev), "Verbose");
 		break;
-	case LOG_WARN:
+	case KCAPI_LOG_WARN:
 		snprintf(sev, sizeof(sev), "Warning");
 		break;
-	case LOG_ERR:
+	case KCAPI_LOG_ERR:
 		snprintf(sev, sizeof(sev), "Error");
 		break;
 	default:
@@ -98,13 +98,13 @@ int _kcapi_common_accept(struct kcapi_handle *handle, int *fdptr)
 
 	fd = accept(handle->tfmfd, NULL, 0);
 	if (fd == -1) {
-		int errsv = 0;
+		int errsv;
 
 		errsv = errno;
-		kcapi_dolog(LOG_ERR, "AF_ALG: accept failed");
+		kcapi_dolog(KCAPI_LOG_ERR, "AF_ALG: accept failed");
 		return -errsv;
 	}
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: accept syscall successful");
+	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: accept syscall successful");
 
 	*fdptr = fd;
 
@@ -115,9 +115,10 @@ int32_t _kcapi_common_send_meta_fd(struct kcapi_handle *handle, int *fdptr,
 				   struct iovec *iov, uint32_t iovlen,
 				   uint32_t enc, uint32_t flags)
 {
-	int32_t ret = -EINVAL;
-	char *buffer = NULL;
-	int errsv = 0;
+	int32_t ret;
+	char buffer_static[80] = { 0 };
+	char *buffer_p = buffer_static;
+	char *buffer_alloc = NULL;
 
 	/* plaintext / ciphertext data */
 	struct cmsghdr *header = NULL;
@@ -146,17 +147,28 @@ int32_t _kcapi_common_send_meta_fd(struct kcapi_handle *handle, int *fdptr,
 
 	memset(&msg, 0, sizeof(msg));
 
-	buffer = calloc(1, bufferlen);
-	if (!buffer)
-		return -ENOMEM;
+	/* allocate buffer, if static buffer is too small */
+	if (bufferlen > sizeof(buffer_static)) {
+		buffer_alloc = calloc(1, bufferlen);
+		if (!buffer_alloc)
+			return -ENOMEM;
+		buffer_p = buffer_alloc;
+		kcapi_dolog(KCAPI_LOG_VERBOSE,
+			    "_kcapi_common_send_meta_fd: submission buffer of size %u allocated",
+			    bufferlen);
+	}
 
-	msg.msg_control = buffer;
+	msg.msg_control = buffer_p;
 	msg.msg_controllen = bufferlen;
 	msg.msg_iov = iov;
 	msg.msg_iovlen = iovlen;
 
 	/* encrypt/decrypt operation */
 	header = CMSG_FIRSTHDR(&msg);
+	if (!header) {
+		ret = -EFAULT;
+		goto out;
+	}
 	header->cmsg_level = SOL_ALG;
 	header->cmsg_type = ALG_SET_OP;
 	header->cmsg_len = CMSG_LEN(sizeof(*type));
@@ -166,6 +178,10 @@ int32_t _kcapi_common_send_meta_fd(struct kcapi_handle *handle, int *fdptr,
 	/* set IV */
 	if (handle->cipher.iv) {
 		header = CMSG_NXTHDR(&msg, header);
+		if (!header) {
+			ret = -EFAULT;
+			goto out;
+		}
 		header->cmsg_level = SOL_ALG;
 		header->cmsg_type = ALG_SET_IV;
 		header->cmsg_len = iv_msg_size;
@@ -178,6 +194,10 @@ int32_t _kcapi_common_send_meta_fd(struct kcapi_handle *handle, int *fdptr,
 	if (handle->aead.assoclen) {
 		/* Set associated data length */
 		header = CMSG_NXTHDR(&msg, header);
+		if (!header) {
+			ret = -EFAULT;
+			goto out;
+		}
 		header->cmsg_level = SOL_ALG;
 		header->cmsg_type = ALG_SET_AEAD_ASSOCLEN;
 		header->cmsg_len = CMSG_LEN(sizeof(*assoclen));
@@ -186,13 +206,16 @@ int32_t _kcapi_common_send_meta_fd(struct kcapi_handle *handle, int *fdptr,
 	}
 
 	ret = sendmsg(*fdptr, &msg, flags);
-	errsv = errno;
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: sendmsg syscall returned %d (errno: %d)",
-		    ret, errsv);
+	if (ret < 0)
+		ret = -errno;
+	kcapi_dolog(KCAPI_LOG_DEBUG,
+		    "AF_ALG: sendmsg syscall returned %d", ret);
 
-	kcapi_memset_secure(buffer, 0, bufferlen);
-	free(buffer);
-	return (ret >= 0) ? ret : -errsv;
+out:
+	kcapi_memset_secure(buffer_p, 0, bufferlen);
+	if (buffer_alloc)
+		free(buffer_alloc);
+	return ret;
 }
 
 int32_t _kcapi_common_send_data_fd(struct kcapi_handle *handle, int *fdptr,
@@ -200,8 +223,7 @@ int32_t _kcapi_common_send_data_fd(struct kcapi_handle *handle, int *fdptr,
 				   uint32_t flags)
 {
 	struct msghdr msg;
-	int32_t ret = 0;
-	int32_t errsv;
+	int32_t ret;
 
 	ret = _kcapi_common_accept(handle, fdptr);
 	if (ret)
@@ -216,11 +238,12 @@ int32_t _kcapi_common_send_data_fd(struct kcapi_handle *handle, int *fdptr,
 	msg.msg_iovlen = iovlen;
 
 	ret = sendmsg(*fdptr, &msg, flags);
-	errsv = errno;
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: sendmsg syscall returned %d (errno: %d)",
-		    ret, errsv);
+	if (ret < 0)
+		ret = -errno;
+	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: sendmsg syscall returned %d",
+		    ret);
 
-	return (ret >= 0) ? ret : -errsv;
+	return ret;
 }
 
 int32_t _kcapi_common_vmsplice_iov_fd(struct kcapi_handle *handle, int *fdptr,
@@ -230,10 +253,12 @@ int32_t _kcapi_common_vmsplice_iov_fd(struct kcapi_handle *handle, int *fdptr,
 	int32_t ret = 0;
 	uint32_t inlen = 0;
 	unsigned long i;
-	int32_t errsv;
 
-	for (i = 0; i < iovlen; i++)
+	for (i = 0; i < iovlen; i++) {
+		if (!(iov + i))
+			return -EINVAL;
 		inlen += iov[i].iov_len;
+	}
 
 	/* kernel processes input data with max size of one page */
 	handle->processed_sg += ((inlen + sysconf(_SC_PAGESIZE) - 1) /
@@ -248,23 +273,27 @@ int32_t _kcapi_common_vmsplice_iov_fd(struct kcapi_handle *handle, int *fdptr,
 		return ret;
 
 	ret = vmsplice(handle->pipes[1], iov, iovlen, SPLICE_F_GIFT|flags);
-	if (0 > ret)
+	if (ret < 0) {
+		ret = -errno;
+		kcapi_dolog(KCAPI_LOG_DEBUG,
+			    "AF_ALG: vmsplice syscall returned %d", ret);
 		return ret;
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: vmsplice syscall returned %d (errno: %d)",
-		    ret, errno);
+	}
+	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: vmsplice syscall returned %d",
+		    ret);
 
 	if ((uint32_t)ret != inlen) {
-		kcapi_dolog(LOG_ERR, "vmsplice: not all data received by kernel (data recieved: %ld -- data sent: %lu)",
+		kcapi_dolog(KCAPI_LOG_ERR, "vmsplice: not all data received by kernel (data recieved: %ld -- data sent: %lu)",
 			(long)ret, (unsigned long)inlen);
 		return -EFAULT;
 	}
 
 	ret = splice(handle->pipes[0], NULL, *fdptr, NULL, ret, flags);
-	errsv = errno;
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: splice syscall returned %d (errno: %d)",
-		    ret, errsv);
+	if (ret < 0)
+		ret = -errno;
+	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: splice syscall returned %d", ret);
 
-	return (ret >= 0) ? ret : -errsv;
+	return ret;
 }
 
 int32_t _kcapi_common_vmsplice_chunk_fd(struct kcapi_handle *handle, int *fdptr,
@@ -273,7 +302,7 @@ int32_t _kcapi_common_vmsplice_chunk_fd(struct kcapi_handle *handle, int *fdptr,
 {
 	struct iovec iov;
 	uint32_t processed = 0;
-	int ret = 0;
+	int32_t ret = 0;
 	uint32_t sflags = (flags & SPLICE_F_MORE) ? MSG_MORE : 0;
 
 	if (inlen > INT_MAX)
@@ -287,8 +316,6 @@ int32_t _kcapi_common_vmsplice_chunk_fd(struct kcapi_handle *handle, int *fdptr,
 		return ret;
 
 	while (inlen) {
-		int32_t ret = 0;
-
 		iov.iov_base = (void*)(uintptr_t)(in + processed);
 		iov.iov_len = inlen;
 
@@ -297,17 +324,29 @@ int32_t _kcapi_common_vmsplice_chunk_fd(struct kcapi_handle *handle, int *fdptr,
 		} else {
 			ret = vmsplice(handle->pipes[1], &iov, 1,
 				       SPLICE_F_GIFT|flags);
-			kcapi_dolog(LOG_DEBUG, "AF_ALG: vmsplice syscall returned %d (errno: %d)",
-				    ret, errno);
-			if (0 > ret)
+			if (ret < 0) {
+				ret = -errno;
+				kcapi_dolog(KCAPI_LOG_DEBUG,
+					    "AF_ALG: vmsplice syscall returned %d",
+					    ret);
 				return ret;
+			}
+			kcapi_dolog(KCAPI_LOG_DEBUG,
+				    "AF_ALG: vmsplice syscall returned %d",
+				    ret);
+
 			ret = splice(handle->pipes[0], NULL, *fdptr, NULL, ret,
 				     flags);
-			kcapi_dolog(LOG_DEBUG, "AF_ALG: splice syscall returned %d (errno: %d)",
-				    ret, errno);
+			if (ret < 0) {
+				ret = -errno;
+				kcapi_dolog(KCAPI_LOG_DEBUG,
+					    "AF_ALG: splice syscall returned %d",
+					    ret);
+				return ret;
+			}
+			kcapi_dolog(KCAPI_LOG_DEBUG,
+				    "AF_ALG: splice syscall returned %d", ret);
 		}
-		if (0 > ret)
-			return ret;
 
 		processed += ret;
 		inlen -= ret;
@@ -378,16 +417,16 @@ static int _kcapi_aio_poll_data(struct kcapi_handle *handle, struct timeval *tv)
 
 	ret = select(efd + 1, &rfds, NULL, NULL, tv);
 	if (ret == -1) {
-		kcapi_dolog(LOG_ERR, "Select Error: %d\n", errno);
+		kcapi_dolog(KCAPI_LOG_ERR, "Select Error: %d\n", errno);
 		return -errno;
 	}
 	if (!FD_ISSET(efd, &rfds)) {
-		kcapi_dolog(LOG_ERR, "aio poll: no FDs\n");
+		kcapi_dolog(KCAPI_LOG_ERR, "aio poll: no FDs\n");
 		return -EFAULT;
 	}
 
 	if (read(efd, &eval, sizeof(eval)) != sizeof(eval)) {
-		kcapi_dolog(LOG_ERR, "efd read error\n");
+		kcapi_dolog(KCAPI_LOG_ERR, "efd read error\n");
 		return -EFAULT;
 	}
 
@@ -401,19 +440,20 @@ int _kcapi_aio_send_iov(struct kcapi_handle *handle, struct iovec *iov,
 			uint32_t iovlen, int access, int enc)
 {
 	int ret;
+	size_t len = iov ? iov->iov_len : 0;
 
 	/*
 	 * Using two syscalls with memcpy is faster than four syscalls
 	 * without memcpy below the given threshold.
 	 */
-	if ((access == KCAPI_ACCESS_HEURISTIC && iov->iov_len <= (1<<13)) ||
+	if ((access == KCAPI_ACCESS_HEURISTIC && len <= (1<<13)) ||
 	    access == KCAPI_ACCESS_SENDMSG) {
 		ret = _kcapi_common_send_meta(handle, iov, iovlen, enc, 0);
 		if (0 > ret)
 			return ret;
 	} else {
 		ret = _kcapi_common_send_meta(handle, NULL, 0, enc,
-					      iov->iov_len ? MSG_MORE : 0);
+					      len ? MSG_MORE : 0);
 		if (0 > ret)
 			return ret;
 		ret = _kcapi_common_vmsplice_iov(handle, iov, iovlen, 0);
@@ -461,10 +501,12 @@ int32_t _kcapi_aio_read_iov_fd(struct kcapi_handle *handle, int *fdptr,
 	ret = io_submit(handle->aio.aio_ctx, iovlen, handle->aio.ciopp);
 	if ((uint32_t)ret != iovlen) {
 		if (ret < 0) {
-			kcapi_dolog(LOG_ERR, "io_read Error: %d\n", errno);
-			return -EFAULT;
+			ret = -errno;
+			kcapi_dolog(KCAPI_LOG_ERR, "io_read Error: %d\n", ret);
+			return ret;
 		} else {
-			kcapi_dolog(LOG_ERR, "Could not sumbit AIO read\n");
+			kcapi_dolog(KCAPI_LOG_ERR,
+				    "Could not sumbit AIO read\n");
 			return -EIO;
 		}
 	}
@@ -476,8 +518,7 @@ int32_t _kcapi_common_recv_data_fd(struct kcapi_handle *handle, int *fdptr,
 				   struct iovec *iov, uint32_t iovlen)
 {
 	struct msghdr msg;
-	int32_t ret = 0;
-	int32_t errsv = 0;
+	int32_t ret;
 
 	ret = _kcapi_common_accept(handle, fdptr);
 	if (ret)
@@ -492,10 +533,10 @@ int32_t _kcapi_common_recv_data_fd(struct kcapi_handle *handle, int *fdptr,
 	msg.msg_iovlen = iovlen;
 
 	ret = recvmsg(*fdptr, &msg, 0);
-	errsv = errno;
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: recvmsg syscall returned %d (errno: %d)",
-		    ret, errsv);
-
+	if (ret < 0)
+		ret = -errno;
+	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: recvmsg syscall returned %d",
+		    ret);
 
 	/*
 	 * As the iovecs are processed and removed from the list in the kernel
@@ -505,7 +546,7 @@ int32_t _kcapi_common_recv_data_fd(struct kcapi_handle *handle, int *fdptr,
 	 * a "valid" error of EBADMSG indicating an integrity error of the
 	 * crypto operation.
 	 */
-	if (ret >= 0 || errsv == EBADMSG)
+	if (ret >= 0 || ret == -EBADMSG)
 		handle->processed_sg = 0;
 
 #if 0
@@ -518,14 +559,13 @@ int32_t _kcapi_common_recv_data_fd(struct kcapi_handle *handle, int *fdptr,
 	}
 #endif
 
-	return (ret >= 0) ? ret : -errsv;
+	return ret;
 }
 
 int32_t _kcapi_common_read_data_fd(struct kcapi_handle *handle, int *fdptr,
 				   uint8_t *out, uint32_t outlen)
 {
-	int32_t ret = 0;
-	int32_t errsv;
+	int32_t ret;
 
 	if (outlen > INT_MAX)
 		return -EMSGSIZE;
@@ -535,25 +575,25 @@ int32_t _kcapi_common_read_data_fd(struct kcapi_handle *handle, int *fdptr,
 		return ret;
 
 	ret = read(*fdptr, out, outlen);
-	errsv = errno;
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: read syscall returned %d (errno: %d)",
-		    ret, errsv);
+	if (ret < 0)
+		ret = -errno;
+	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: read syscall returned %d", ret);
 
-	return (ret >= 0) ? ret : -errsv;
+	return ret;
 }
 
 int _kcapi_common_setkey(struct kcapi_handle *handle,
 			 const uint8_t *key, uint32_t keylen)
 {
-	int ret = 0;
-	int errsv;
+	int ret;
 
 	ret = setsockopt(handle->tfmfd, SOL_ALG, ALG_SET_KEY, key, keylen);
-	errsv = errno;
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: sendmsg syscall returned %d (errno: %d)",
-		    ret, errsv);
+	if (ret < 0)
+		ret = -errno;
+	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: sendmsg syscall returned %d",
+		    ret);
 
-	return (ret >= 0) ? ret : -errsv;
+	return ret;
 }
 
 static int __kcapi_common_getinfo(struct kcapi_handle *handle,
@@ -561,7 +601,6 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 				  int drivername)
 {
 	int ret = -EFAULT;
-	int errsv = 0;
 
 	/* NETLINK_CRYPTO specific */
 	char buf[4096];
@@ -570,7 +609,7 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 		struct nlmsghdr n;
 		struct crypto_user_alg cru;
 	} req;
-	struct crypto_user_alg *cru_res;
+	struct crypto_user_alg *cru_res = NULL;
 	int res_len = 0;
 	struct rtattr *tb[CRYPTOCFGA_MAX+1];
 	struct rtattr *rta;
@@ -600,33 +639,36 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 	/* talk to netlink socket */
 	sd =  socket(AF_NETLINK, SOCK_RAW, NETLINK_CRYPTO);
 	if (sd < 0) {
-		kcapi_dolog(LOG_ERR, "Netlink error: cannot open netlink socket");
+		kcapi_dolog(KCAPI_LOG_ERR,
+			    "Netlink error: cannot open netlink socket");
 		return -errno;
 	}
 	memset(&nl, 0, sizeof(nl));
 	nl.nl_family = AF_NETLINK;
 	if (bind(sd, (struct sockaddr*)&nl, sizeof(nl)) < 0) {
-		errsv = errno;
-		kcapi_dolog(LOG_ERR, "Netlink error: cannot bind netlink socket");
+		ret = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR,
+			    "Netlink error: cannot bind netlink socket");
 		goto out;
 	}
 	/* sanity check that netlink socket was successfully opened */
 	addr_len = sizeof(nl);
 	if (getsockname(sd, (struct sockaddr*)&nl, &addr_len) < 0) {
-		errsv = errno;
-		kcapi_dolog(LOG_ERR, "Netlink error: cannot getsockname");
+		ret = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR, "Netlink error: cannot getsockname");
 		goto out;
 	}
 	if (addr_len != sizeof(nl)) {
-		errsv = errno;
-		kcapi_dolog(LOG_ERR, "Netlink error: wrong address length %d",
-			addr_len);
+		ret = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR,
+			    "Netlink error: wrong address length %d", addr_len);
 		goto out;
 	}
 	if (nl.nl_family != AF_NETLINK) {
-		errsv = errno;
-		kcapi_dolog(LOG_ERR, "Netlink error: wrong address family %d",
-			nl.nl_family);
+		ret = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR,
+			    "Netlink error: wrong address family %d",
+			    nl.nl_family);
 		goto out;
 	}
 
@@ -640,8 +682,8 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 	msg.msg_iov = &iov;
 	msg.msg_iovlen = 1;
 	if (sendmsg(sd, &msg, 0) < 0) {
-		errsv = errno;
-		kcapi_dolog(LOG_ERR, "Netlink error: sendmsg failed");
+		ret = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR, "Netlink error: sendmsg failed");
 		goto out;
 	}
 	memset(buf,0,sizeof(buf));
@@ -652,18 +694,20 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 		if (ret < 0) {
 			if (errno == EINTR || errno == EAGAIN)
 				continue;
-			errsv = errno;
-			kcapi_dolog(LOG_ERR, "Netlink error: netlink receive error");
+			ret = -errno;
+			kcapi_dolog(KCAPI_LOG_ERR,
+				    "Netlink error: netlink receive error");
 			goto out;
 		}
 		if (ret == 0) {
-			errsv = errno;
-			kcapi_dolog(LOG_ERR, "Netlink error: no data");
+			ret = -errno;
+			kcapi_dolog(KCAPI_LOG_ERR, "Netlink error: no data");
 			goto out;
 		}
 		if ((uint32_t)ret > sizeof(buf)) {
-			errsv = errno;
-			kcapi_dolog(LOG_ERR, "Netlink error: received too much data");
+			ret = -errno;
+			kcapi_dolog(KCAPI_LOG_ERR,
+				    "Netlink error: received too much data");
 			goto out;
 		}
 		break;
@@ -686,11 +730,16 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 		res_len -= NLMSG_SPACE(sizeof(*cru_res));
 	}
 	if (res_len < 0) {
-		kcapi_dolog(LOG_ERR, "Netlink error: nlmsg len %d", res_len);
+		kcapi_dolog(KCAPI_LOG_ERR, "Netlink error: nlmsg len %d",
+			    res_len);
 		goto out;
 	}
 
 	/* parse data */
+	if (!cru_res) {
+		ret = -EFAULT;
+		goto out;
+	}
 	rta = CR_RTA(cru_res);
 	memset(tb, 0, sizeof(struct rtattr *) * (CRYPTOCFGA_MAX + 1));
 	while (RTA_OK(rta, res_len)) {
@@ -699,7 +748,7 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 		rta = RTA_NEXT(rta, res_len);
 	}
 	if (res_len) {
-		kcapi_dolog(LOG_ERR, "Netlink error: unprocessed data %d",
+		kcapi_dolog(KCAPI_LOG_ERR, "Netlink error: unprocessed data %d",
 			    res_len);
 		goto out;
 	}
@@ -710,7 +759,7 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 			(struct crypto_report_hash *) RTA_DATA(rta);
 		handle->info.hash_digestsize = rsh->digestsize;
 		handle->info.blocksize = rsh->blocksize;
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: hash with digestsize %u,  blocksize %u",
+		kcapi_dolog(KCAPI_LOG_DEBUG, "Get cipher info: hash with digestsize %u,  blocksize %u",
 			    rsh->digestsize, rsh->blocksize);
 	}
 	if (tb[CRYPTOCFGA_REPORT_BLKCIPHER]) {
@@ -721,7 +770,7 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 		handle->info.ivsize = rblk->ivsize;
 		handle->info.blk_min_keysize = rblk->min_keysize;
 		handle->info.blk_max_keysize = rblk->max_keysize;
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: block cipher with blocksize %u, ivsize %u, minimum keysize %u, maximum keysize %u",
+		kcapi_dolog(KCAPI_LOG_DEBUG, "Get cipher info: block cipher with blocksize %u, ivsize %u, minimum keysize %u, maximum keysize %u",
 			    rblk->blocksize, rblk->ivsize, rblk->min_keysize,
 			    rblk->max_keysize);
 	}
@@ -732,7 +781,7 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 		handle->info.blocksize = raead->blocksize;
 		handle->info.ivsize = raead->ivsize;
 		handle->info.aead_maxauthsize = raead->maxauthsize;
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: AEAD block cipher with blocksize %u, ivsize %u, maximum authentication size %u",
+		kcapi_dolog(KCAPI_LOG_DEBUG, "Get cipher info: AEAD block cipher with blocksize %u, ivsize %u, maximum authentication size %u",
 			    raead->blocksize, raead->ivsize,
 			    raead->maxauthsize);
 	}
@@ -741,33 +790,42 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 		struct crypto_report_rng *rrng =
 			(struct crypto_report_rng *) RTA_DATA(rta);
 		handle->info.rng_seedsize = rrng->seedsize;
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: RNG cipher with seedsize %u",
+		kcapi_dolog(KCAPI_LOG_DEBUG, "Get cipher info: RNG cipher with seedsize %u",
 			    rrng->seedsize);
 	}
 	if (tb[CRYPTOCFGA_UNSPEC])
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: unspecified data received");
+		kcapi_dolog(KCAPI_LOG_DEBUG,
+			    "Get cipher info: unspecified data received");
 	if (tb[CRYPTOCFGA_PRIORITY_VAL])
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: u32 value received");
+		kcapi_dolog(KCAPI_LOG_DEBUG,
+			    "Get cipher info: u32 value received");
 	if (tb[CRYPTOCFGA_REPORT_LARVAL])
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: larval value received");
+		kcapi_dolog(KCAPI_LOG_DEBUG,
+			    "Get cipher info: larval value received");
 	if (tb[CRYPTOCFGA_REPORT_COMPRESS])
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: compression algorithm type received");
+		kcapi_dolog(KCAPI_LOG_DEBUG,
+			    "Get cipher info: compression algorithm type received");
 	if (tb[CRYPTOCFGA_REPORT_CIPHER])
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: simple cipher algorithm type received");
+		kcapi_dolog(KCAPI_LOG_DEBUG,
+			    "Get cipher info: simple cipher algorithm type received");
 	if (tb[CRYPTOCFGA_REPORT_AKCIPHER])
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: asymmetric cipher algorithm type received");
+		kcapi_dolog(KCAPI_LOG_DEBUG,
+			    "Get cipher info: asymmetric cipher algorithm type received");
 	if (tb[CRYPTOCFGA_REPORT_KPP])
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: kpp cipher algorithm type received");
+		kcapi_dolog(KCAPI_LOG_DEBUG,
+			    "Get cipher info: kpp cipher algorithm type received");
 	if (tb[CRYPTOCFGA_REPORT_ACOMP])
-		kcapi_dolog(LOG_DEBUG, "Get cipher info: asymmetric compression algorithm type received");
-	kcapi_dolog(LOG_VERBOSE, "Get cipher info: all information for %s received from kernel",
+		kcapi_dolog(KCAPI_LOG_DEBUG,
+			    "Get cipher info: asymmetric compression algorithm type received");
+	kcapi_dolog(KCAPI_LOG_VERBOSE,
+		    "Get cipher info: all information for %s received from kernel",
 		    ciphername);
 
 	ret = 0;
 
 out:
 	close(sd);
-	return (errsv) ? -errsv : ret;
+	return ret;
 }
 
 static int _kcapi_common_getinfo(struct kcapi_handle *handle,
@@ -781,7 +839,7 @@ static int _kcapi_common_getinfo(struct kcapi_handle *handle,
 
 static inline void _kcapi_aio_destroy(struct kcapi_handle *handle)
 {
-	if (handle->aio.disable)
+	if (handle->aio.disable == true)
 		return;
 	if (handle->aio.efd != -1)
 		close(handle->aio.efd);
@@ -794,7 +852,7 @@ static inline void _kcapi_aio_destroy(struct kcapi_handle *handle)
 	handle->aio.ciopp = NULL;
 }
 
-void _kcapi_handle_destroy(struct kcapi_handle *handle)
+void _kcapi_handle_destroy_nofree(struct kcapi_handle *handle)
 {
 	if (!handle)
 		return;
@@ -808,6 +866,11 @@ void _kcapi_handle_destroy(struct kcapi_handle *handle)
 		close(handle->pipes[1]);
 	_kcapi_aio_destroy(handle);
 	kcapi_memset_secure(handle, 0, sizeof(struct kcapi_handle));
+}
+
+void _kcapi_handle_destroy(struct kcapi_handle *handle)
+{
+	_kcapi_handle_destroy_nofree(handle);
 	free(handle);
 }
 
@@ -818,46 +881,46 @@ static int _kcapi_get_kernver(struct kcapi_handle *handle)
 	char *res = NULL;
 
 	if (uname(&kernel))
-		return errno;
+		return -errno;
 
 	/* 3.15.0 */
 	res = strtok_r(kernel.release, ".", &saveptr);
 	if (!res) {
 		printf("Could not parse kernel version");
-		return EFAULT;
+		return -EFAULT;
 	}
 	handle->sysinfo.kernel_maj = strtoul(res, NULL, 10);
 	res = strtok_r(NULL, ".", &saveptr);
 	if (!res) {
 		printf("Could not parse kernel version");
-		return EFAULT;
+		return -EFAULT;
 	}
 	handle->sysinfo.kernel_minor = strtoul(res, NULL, 10);
 	res = strtok_r(NULL, ".", &saveptr);
 	if (!res) {
 		printf("Could not parse kernel version");
-		return EFAULT;
+		return -EFAULT;
 	}
 	handle->sysinfo.kernel_patchlevel = strtoul(res, NULL, 10);
 
 	return 0;
 }
 
-/* return 1 if kernel is greater or equal to given values, otherwise 0 */
-static int _kcapi_kernver_ge(struct kcapi_handle *handle, unsigned int maj,
-			     unsigned int minor, unsigned int patchlevel)
+/* return true if kernel is greater or equal to given values, otherwise false */
+static bool _kcapi_kernver_ge(struct kcapi_handle *handle, unsigned int maj,
+			      unsigned int minor, unsigned int patchlevel)
 {
 	if (maj < handle->sysinfo.kernel_maj)
-		return 1;
+		return true;
 	if (maj == handle->sysinfo.kernel_maj) {
 		if (minor < handle->sysinfo.kernel_minor)
-			return 1;
+			return true;
 		if (minor == handle->sysinfo.kernel_minor) {
 			if (patchlevel <= handle->sysinfo.kernel_patchlevel)
-				return 1;
+				return true;
 		}
 	}
-	return 0;
+	return false;
 }
 
 static int _kcapi_aio_init(struct kcapi_handle *handle, const char *type)
@@ -866,39 +929,29 @@ static int _kcapi_aio_init(struct kcapi_handle *handle, const char *type)
 	int err;
 
 	if (!strncmp("aead", type, 4)) {
-		if (!_kcapi_kernver_ge(handle, 4, 13, 0)) {
-			kcapi_dolog(LOG_VERBOSE, "AIO support for AEAD cipher not present on current kernel\n");
-			err = EOPNOTSUPP;
+		if (!_kcapi_kernver_ge(handle, 4, 7, 0)) {
+			kcapi_dolog(KCAPI_LOG_VERBOSE, "AIO support for AEAD cipher not present on current kernel");
+			err = -EOPNOTSUPP;
 			goto err;
 		}
-	} else if (!strncmp("skcipher", type, 8)) {
-		if (!_kcapi_kernver_ge(handle, 4, 13, 0)) {
-			kcapi_dolog(LOG_VERBOSE, "AIO support for symmetric ciphers not present on current kernel\n");
-			err = EOPNOTSUPP;
+	}
+	if (!strncmp("skcipher", type, 8)) {
+		if (!_kcapi_kernver_ge(handle, 4, 1, 0)) {
+			kcapi_dolog(KCAPI_LOG_VERBOSE, "AIO support for symmetric ciphers not present on current kernel");
+			err = -EOPNOTSUPP;
 			goto err;
 		}
-	} else if (!strncmp("akcipher", type, 8)) {
-		if (!_kcapi_kernver_ge(handle, 4, 99, 0)) {
-			kcapi_dolog(LOG_VERBOSE, "AIO support for asymmetric ciphers not present on current kernel\n");
-			err = EOPNOTSUPP;
-			goto err;
-		}
-	} else {
-		kcapi_dolog(LOG_WARN, "AIO support for unknown cipher type %s not present\n",
-			    type);
-		err = EFAULT;
-		goto err;
 	}
 
 	handle->aio.cio = calloc(KCAPI_AIO_CONCURRENT, sizeof(struct iocb));
 	if (!handle->aio.cio) {
-		err = ENOMEM;
+		err = -ENOMEM;
 		goto err;
 	}
 
 	handle->aio.ciopp = calloc(KCAPI_AIO_CONCURRENT, sizeof(void *));
 	if (!handle->aio.ciopp) {
-		err = ENOMEM;
+		err = -ENOMEM;
 		goto err;
 	}
 
@@ -911,26 +964,26 @@ static int _kcapi_aio_init(struct kcapi_handle *handle, const char *type)
 
 	handle->aio.efd = eventfd(0, EFD_CLOEXEC);
 	if (handle->aio.efd < 0) {
-		err = errno;
-		kcapi_dolog(LOG_ERR, "Event FD cannot be initialized: %d\n",
+		err = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR, "Event FD cannot be initialized: %d\n",
 			    err);
 		goto err;
 	}
 
 	err = io_setup(KCAPI_AIO_CONCURRENT, &handle->aio.aio_ctx);
 	if (err < 0) {
-		kcapi_dolog(LOG_ERR, "io_setup error %d\n", err);
-		/* turn return code into an errno */
-		err = -err;
+		err = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR, "io_setup error %d\n", err);
 		goto err;
 	}
 
-	kcapi_dolog(LOG_VERBOSE, "asynchronous I/O initialized");
+	handle->aio.disable = false;
+	kcapi_dolog(KCAPI_LOG_VERBOSE, "asynchronous I/O initialized");
 
 	return 0;
 
 err:
-	handle->aio.disable = 1;
+	handle->aio.disable = true;
 	if (handle->aio.efd != -1)
 		close(handle->aio.efd);
 	handle->aio.efd = -1;
@@ -953,32 +1006,27 @@ static void _kcapi_handle_flags(struct kcapi_handle *handle)
 				      UINT_MAX : ALG_MAX_PAGES;
 }
 
-int _kcapi_handle_init(struct kcapi_handle **caller, const char *type,
-		       const char *ciphername, uint32_t flags)
+int _kcapi_allocated_handle_init(struct kcapi_handle *handle, const char *type,
+				 const char *ciphername, uint32_t flags)
 {
 	struct sockaddr_alg sa;
-	struct kcapi_handle *handle;
 	int ret;
-	int errsv = 0;
 	char versionbuffer[50];
 
 	kcapi_versionstring(versionbuffer, sizeof(versionbuffer));
-	kcapi_dolog(LOG_VERBOSE, "%s - initializing cipher operation with kernel",
+	kcapi_dolog(KCAPI_LOG_VERBOSE,
+		    "%s - initializing cipher operation with kernel",
 		    versionbuffer);
 
-	handle = calloc(1, sizeof(struct kcapi_handle));
-	if (!handle)
-		return -ENOMEM;
-
 	handle->opfd = -1;
 	handle->tfmfd = -1;
 	handle->pipes[0] = -1;
 	handle->pipes[1] = -1;
 	handle->aio.efd = -1;
 
-	errsv = _kcapi_get_kernver(handle);
-	if (errsv)
-		goto err;
+	ret = _kcapi_get_kernver(handle);
+	if (ret)
+		return ret;
 
 	memset(&sa, 0, sizeof(sa));
 	sa.salg_family = AF_ALG;
@@ -987,42 +1035,42 @@ int _kcapi_handle_init(struct kcapi_handle **caller, const char *type,
 
 	handle->tfmfd = socket(AF_ALG, SOCK_SEQPACKET, 0);
 	if (handle->tfmfd == -1) {
-		errsv = errno;
-		kcapi_dolog(LOG_ERR, "AF_ALG: socket syscall failed (errno: %d)",
-			    errsv);
-		goto err;
+		ret = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR,
+			    "AF_ALG: socket syscall failed (errno: %d)",
+			    ret);
+		return ret;
 	}
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: socket syscall passed (errno: %d)",
-		    errno);
+	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: socket syscall passed");
 
 	if (bind(handle->tfmfd, (struct sockaddr *)&sa, sizeof(sa)) == -1) {
-		errsv = errno;
-		kcapi_dolog(LOG_ERR, "AF_ALG: bind failed (errno: %d)",
-			    errsv);
-		goto err;
+		ret = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR, "AF_ALG: bind failed (errno: %d)",
+			    ret);
+		return ret;
 	}
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: bind syscall passed (errno: %d)",
-		    errno);
+	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: bind syscall passed");
 
 	ret = pipe(handle->pipes);
 	if (ret) {
-		errsv = errno;
-		kcapi_dolog(LOG_ERR, "AF_ALG: pipe syscall failed (errno: %d)",
-			    errsv);
-		goto err;
+		ret = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR,
+			    "AF_ALG: pipe syscall failed (errno: %d)",
+			    ret);
+		return ret;
 	}
-	kcapi_dolog(LOG_DEBUG, "AF_ALG: pipe syscall passed");
+	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: pipe syscall passed");
 
 	ret = _kcapi_common_getinfo(handle, ciphername);
 	if (ret) {
-		errsv = errno;
-		kcapi_dolog(LOG_ERR, "NETLINK_CRYPTO: cannot obtain cipher information for %s (is required crypto_user.c patch missing? see documentation)",
+		ret = -errno;
+		kcapi_dolog(KCAPI_LOG_ERR, "NETLINK_CRYPTO: cannot obtain cipher information for %s (is required crypto_user.c patch missing? see documentation)",
 			    ciphername);
-		goto err;
+		return ret;
 	}
 
 	if (flags & KCAPI_INIT_AIO) {
-		errsv = _kcapi_aio_init(handle, type);
+		ret = _kcapi_aio_init(handle, type);
 
 		/*
 		 * We complain about kernels without AIO support, but allow
@@ -1031,23 +1079,38 @@ int _kcapi_handle_init(struct kcapi_handle **caller, const char *type,
 		 * libkcapi AIO API even though the kernel does not support it.
 		 * This allows compatibility for applications.
 		 */
-		if (errsv && errsv != EOPNOTSUPP)
-			goto err;
+		if (ret && ret != -EOPNOTSUPP)
+			return ret;
+		ret = 0;
 	} else
-		handle->aio.disable = 1;
+		handle->aio.disable = true;
 
 	_kcapi_handle_flags(handle);
 
-	kcapi_dolog(LOG_VERBOSE, "communication for %s with kernel initialized",
+	kcapi_dolog(KCAPI_LOG_VERBOSE,
+		    "communication for %s with kernel initialized",
 		    ciphername);
 
-	*caller = handle;
+	return ret;
+}
+
+int _kcapi_handle_init(struct kcapi_handle **caller, const char *type,
+		       const char *ciphername, uint32_t flags)
+{
+	struct kcapi_handle *handle;
+	int ret;
+
+	handle = calloc(1, sizeof(struct kcapi_handle));
+	if (!handle)
+		return -ENOMEM;
+
+	ret = _kcapi_allocated_handle_init(handle, type, ciphername, flags);
+	if (ret)
+		_kcapi_handle_destroy(handle);
+	else
+		*caller = handle;
 
 	return ret;
-
-err:
-	_kcapi_handle_destroy(handle);
-	return -errsv;
 }
 
 int32_t _kcapi_cipher_crypt(struct kcapi_handle *handle, const uint8_t *in,
@@ -1129,12 +1192,12 @@ int32_t _kcapi_cipher_crypt_aio(struct kcapi_handle *handle,
 				struct iovec *iniov, struct iovec *outiov,
 				uint32_t iovlen, int access, int enc)
 {
-	int32_t ret = 0;
-	int32_t rc;
+	int32_t processed = 0;
+	int32_t ret;
 	uint32_t tosend = iovlen;
 
-	if (handle->aio.disable) {
-		kcapi_dolog(LOG_WARN, "AIO support disabled\n");
+	if (handle->aio.disable == true) {
+		kcapi_dolog(KCAPI_LOG_WARN, "AIO support disabled\n");
 		return -EOPNOTSUPP;
 	}
 
@@ -1144,19 +1207,18 @@ int32_t _kcapi_cipher_crypt_aio(struct kcapi_handle *handle,
 	while (tosend) {
 		uint32_t process = (KCAPI_AIO_CONCURRENT < tosend) ?
 					KCAPI_AIO_CONCURRENT : tosend;
-		int32_t rc = _kcapi_aio_send_iov(handle, iniov, process,
-						 access, enc);
 
-		if (rc < 0)
-			return rc;
+		ret = _kcapi_aio_send_iov(handle, iniov, process, access, enc);
+		if (ret < 0)
+			return ret;
 
-		rc = _kcapi_aio_read_iov(handle, outiov, process);
-		if (rc < 0)
-			return rc;
+		ret = _kcapi_aio_read_iov(handle, outiov, process);
+		if (ret < 0)
+			return ret;
 
 		iniov += process;
 		outiov += process;
-		ret += rc;
+		processed += ret;
 		tosend = iovlen - handle->aio.completed_reads;
 	}
 
@@ -1173,11 +1235,11 @@ int32_t _kcapi_cipher_crypt_aio(struct kcapi_handle *handle,
 	 * "open-ended" multi-staged AIO operation could not be implemented.
 	 */
 
-	rc = _kcapi_aio_read_all(handle, iovlen - handle->aio.completed_reads,
-				 NULL);
-	if (rc < 0)
-		return rc;
-	ret += rc;
+	ret = _kcapi_aio_read_all(handle, iovlen - handle->aio.completed_reads,
+				  NULL);
+	if (ret < 0)
+		return ret;
+	processed += ret;
 
-	return ret;
+	return processed;
 }
