@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 - 2016, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2015 - 2017, Stephan Mueller <smueller@chronox.de>
  *
  * License: see COPYING file in root directory
  *
@@ -331,6 +331,11 @@ int32_t kcapi_cipher_stream_init_dec(struct kcapi_handle *handle,
  * rule is not obeyed, the thread invoking kcapi_cipher_stream_update()
  * will be put to sleep until another thread invokes kcapi_cipher_stream_op().
  *
+ * WARNING: The memory referenced by @iov is not accessed by the kernel
+ * during this call. The memory is first accessed when kcapi_cipher_stream_op()
+ * is called. Thus, you MUST make sure that the referenced memory is still
+ * present at the time kcapi_cipher_stream_op() is called.
+ *
  * @return number of bytes sent to the kernel upon success;
  *	   a negative errno-style error code if an error occurred
  */
@@ -362,6 +367,146 @@ int32_t kcapi_cipher_stream_update(struct kcapi_handle *handle,
 int32_t kcapi_cipher_stream_op(struct kcapi_handle *handle,
 			       struct iovec *iov, uint32_t iovlen);
 
+/**
+ * kcapi_cipher_enc_aes_cbc - Convenience function for AES CBC encryption
+ *
+ * @key: [in] key buffer
+ * @keylen: [in] length of key buffer
+ * @in: [in] plaintext data buffer
+ * @inlen: [in] length of in buffer
+ * @iv: [in] IV to be used for cipher operation
+ * @out: [out] ciphertext data buffer
+ * @outlen: [in] length of out buffer
+ *
+ * The convenience function performs an AES CBC encryption operation
+ * using the provided key, the given input buffer and the given IV.
+ * The output is stored in the out buffer.
+ *
+ * Note, AES CBC requires an input data that is a multiple of 16 bytes.
+ * If you have data that is not guaranteed to be multiples of 16 bytes, either
+ * add zero bytes at the end of the buffer to pad it up to a multiple of 16
+ * bytes. Otherwise, the CTR mode encryption operation may be usable.
+ *
+ * The output buffer must be at least as large as the input buffer.
+ *
+ * The IV must be exactly 16 bytes in size.
+ *
+ * The AES type (AES-128, AES-192 or AES-256) is determined by the size
+ * of the given key. If the key is 16 bytes long, AES-128 is used. A 24 byte
+ * key implies AES-192 and a 32 byte key implies AES-256.
+ *
+ * @return number of bytes generated upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_cipher_enc_aes_cbc(const uint8_t *key, uint32_t keylen,
+				 const uint8_t *in, uint32_t inlen,
+				 const uint8_t *iv,
+				 uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_cipher_dec_aes_cbc - Convenience function for AES CBC decryption
+ *
+ * @key: [in] key buffer
+ * @keylen: [in] length of key buffer
+ * @in: [in] ciphertext data buffer
+ * @inlen: [in] length of in buffer
+ * @iv: [in] IV to be used for cipher operation
+ * @out: [out] plaintext data buffer
+ * @outlen: [in] length of out buffer
+ *
+ * The convenience function performs an AES CBC decryption operation
+ * using the provided key, the given input buffer and the given IV.
+ * The output is stored in the out buffer.
+ *
+ * Note, AES CBC requires an input data that is a multiple of 16 bytes.
+ * If you have data that is not guaranteed to be multiples of 16 bytes, either
+ * add zero bytes at the end of the buffer to pad it up to a multiple of 16
+ * bytes. Otherwise, the CTR mode encryption operation may be usable.
+ *
+ * The output buffer must be at least as large as the input buffer.
+ *
+ * The IV must be exactly 16 bytes in size.
+ *
+ * The AES type (AES-128, AES-192 or AES-256) is determined by the size
+ * of the given key. If the key is 16 bytes long, AES-128 is used. A 24 byte
+ * key implies AES-192 and a 32 byte key implies AES-256.
+ *
+ * @return number of bytes generated upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_cipher_dec_aes_cbc(const uint8_t *key, uint32_t keylen,
+				 const uint8_t *in, uint32_t inlen,
+				 const uint8_t *iv,
+				 uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_cipher_enc_aes_ctr - Convenience function for AES CTR encryption
+ *
+ * @key: [in] key buffer
+ * @keylen: [in] length of key buffer
+ * @in: [in] plaintext data buffer
+ * @inlen: [in] length of in buffer
+ * @ctr: [in] start counter value to be used for cipher operation
+ * @out: [out] ciphertext data buffer
+ * @outlen: [in] length of out buffer
+ *
+ * The convenience function performs an AES counter mode encryption operation
+ * using the provided key, the given input buffer and the given IV.
+ * The output is stored in the out buffer.
+ *
+ * The input buffer can be of arbitrary length.
+ *
+ * The output buffer must be at least as large as the input buffer.
+ *
+ * The start counter can contain all zeros (not a NULL buffer!) and must be
+ * exactly 16 bytes in size.
+ *
+ * The AES type (AES-128, AES-192 or AES-256) is determined by the size
+ * of the given key. If the key is 16 bytes long, AES-128 is used. A 24 byte
+ * key implies AES-192 and a 32 byte key implies AES-256.
+ *
+ * @return number of bytes generated upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_cipher_enc_aes_ctr(const uint8_t *key, uint32_t keylen,
+				 const uint8_t *in, uint32_t inlen,
+				 const uint8_t *ctr,
+				 uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_cipher_dec_aes_ctr - Convenience function for AES CTR decryption
+ *
+ * @key: [in] key buffer
+ * @keylen: [in] length of key buffer
+ * @in: [in] ciphertext data buffer
+ * @inlen: [in] length of in buffer
+ * @ctr: [in] start counter value to be used for cipher operation
+ * @out: [out] plaintext data buffer
+ * @outlen: [in] length of out buffer
+ *
+ * The convenience function performs an AES counter mode encryption operation
+ * using the provided key, the given input buffer and the given IV.
+ * The output is stored in the out buffer.
+ *
+ * The input buffer can be of arbitrary length.
+ *
+ * The output buffer must be at least as large as the input buffer.
+ *
+ * The start counter can contain all zeros (not a NULL buffer!) and must be
+ * exactly 16 bytes in size.
+ *
+ * The AES type (AES-128, AES-192 or AES-256) is determined by the size
+ * of the given key. If the key is 16 bytes long, AES-128 is used. A 24 byte
+ * key implies AES-192 and a 32 byte key implies AES-256.
+ *
+ * @return number of bytes generated upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_cipher_dec_aes_ctr(const uint8_t *key, uint32_t keylen,
+				 const uint8_t *in, uint32_t inlen,
+				 const uint8_t *ctr,
+				 uint8_t *out, uint32_t outlen);
+
 /**
  * kcapi_cipher_ivsize() - return size of IV required for cipher
  *
@@ -551,39 +696,6 @@ int32_t kcapi_aead_encrypt_aio(struct kcapi_handle *handle, struct iovec *iniov,
 			       struct iovec *outiov, uint32_t iovlen,
 			       const uint8_t *iv, int access);
 
-/**
- * kcapi_aead_getdata() - get the resulting data from encryption
- *
- * @handle: [in] cipher handle
- * @encdata: [in] data buffer returned by the encryption operation
- * @encdatalen: [in] size of the encryption data buffer
- * @aad: [out] AD buffer pointer;  when set to NULL, no data pointer is
- *	returned
- * @aadlen: [out] length of AD; when aad was set to NULL, no information is
- *	returned
- * @data: [out] pointer to output buffer from AEAD encryption operation
- *	when set to NULL, no data pointer is returned
- * @datalen: [out] length of data buffer; when data was set to NULL, no
- *	information is returned
- * @tag: [out] tag buffer pointer;  when set to NULL, no data pointer is
- *	returned
- * @taglen: [out] length of tag; when tag was set to NULL, no information
- *	is returned
- *
- * DEPRECATED since 0.13.0
- *
- * This function is a service function to the consumer to locate the right
- * ciphertext buffer offset holding the authentication tag. In addition, it
- * provides the consumer with the length of the tag and the length of the
- * ciphertext.
- */
-DSO_DEPRECATED("0.13.0")
-void kcapi_aead_getdata(struct kcapi_handle *handle,
-			uint8_t *encdata, uint32_t encdatalen,
-			uint8_t **aad, uint32_t *aadlen,
-			uint8_t **data, uint32_t *datalen,
-			uint8_t **tag, uint32_t *taglen);
-
 /**
  * kcapi_aead_getdata_input() - get the pointers into input buffer
  *
@@ -843,6 +955,11 @@ int32_t kcapi_aead_stream_init_dec(struct kcapi_handle *handle,
  * sysconf(_SC_PAGESIZE) * ALG_MAX_PAGES at one time. Longer input data cannot
  * be handled by the kernel.
  *
+ * WARNING: The memory referenced by @iov is not accessed by the kernel
+ * during this call. The memory is first accessed when kcapi_cipher_stream_op()
+ * is called. Thus, you MUST make sure that the referenced memory is still
+ * present at the time kcapi_cipher_stream_op() is called.
+ *
  * @return number of bytes sent to the kernel upon success;
  *	   a negative errno-style error code if an error occurred
  */
@@ -864,6 +981,11 @@ int32_t kcapi_aead_stream_update(struct kcapi_handle *handle,
  * exception that it marks the last data buffer before the cipher operation
  * is triggered. Typically, the tag value is provided with this call.
  *
+ * WARNING: The memory referenced by @iov is not accessed by the kernel
+ * during this call. The memory is first accessed when kcapi_cipher_stream_op()
+ * is called. Thus, you MUST make sure that the referenced memory is still
+ * present at the time kcapi_cipher_stream_op() is called.
+ *
  * @return number of bytes sent to the kernel upon success;
  *	   a negative errno-style error code if an error occurred
  */
@@ -926,22 +1048,6 @@ uint32_t kcapi_aead_blocksize(struct kcapi_handle *handle);
  */
 uint32_t kcapi_aead_authsize(struct kcapi_handle *handle);
 
-/**
- * kcapi_aead_outbuflen() - return minimum output buffer length
- *
- * @handle: [in] cipher handle
- * @inlen: [in] size of plaintext or size of ciphertext
- * @assoclen: [in] size of associated data (AD)
- * @taglen: [in] size of authentication tag
- *
- * DEPRECATED since 0.13.0
- *
- * @return minimum size of output data length in bytes
- */
-DSO_DEPRECATED("0.13.0")
-uint32_t kcapi_aead_outbuflen(struct kcapi_handle *handle, uint32_t inlen,
-			      uint32_t assoclen, uint32_t taglen);
-
 /**
  * kcapi_aead_inbuflen_enc() - return minimum encryption input buffer length
  *
@@ -1123,6 +1229,226 @@ int32_t kcapi_md_digest(struct kcapi_handle *handle,
 		       const uint8_t *in, uint32_t inlen,
 		       uint8_t *out, uint32_t outlen);
 
+/**
+ * kcapi_md_sha1 - SHA-1 message digest on one buffer
+ *
+ * @in: [in] buffer with input data
+ * @inlen: [in] length of input buffer
+ * @out: [out] buffer for message digest
+ * @outlen: [in] length of out
+ *
+ * With this one-shot convenience function, a message digest of the given buffer
+ * is generated. The output buffer must be allocated by the caller and have at
+ * least the length of the message digest size for the chosen message digest.
+ *
+ * @return size of message digest upon success;
+ *	    -EIO - data cannot be obtained;
+ * 	    -ENOMEM - buffer is too small for the complete message digest,
+ * 	    the buffer is filled with the truncated message digest
+ */
+int32_t kcapi_md_sha1(const uint8_t *in, uint32_t inlen,
+		      uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_md_sha224 - SHA-224 message digest on one buffer
+ *
+ * @in: [in] buffer with input data
+ * @inlen: [in] length of input buffer
+ * @out: [out] buffer for message digest
+ * @outlen: [in] length of out
+ *
+ * With this one-shot convenience function, a message digest of the given buffer
+ * is generated. The output buffer must be allocated by the caller and have at
+ * least the length of the message digest size for the chosen message digest.
+ *
+ * @return size of message digest upon success;
+ *	    -EIO - data cannot be obtained;
+ * 	    -ENOMEM - buffer is too small for the complete message digest,
+ * 	    the buffer is filled with the truncated message digest
+ */
+int32_t kcapi_md_sha224(const uint8_t *in, uint32_t inlen,
+			uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_md_sha256 - SHA-256 message digest on one buffer
+ *
+ * @in: [in] buffer with input data
+ * @inlen: [in] length of input buffer
+ * @out: [out] buffer for message digest
+ * @outlen: [in] length of out
+ *
+ * With this one-shot convenience function, a message digest of the given buffer
+ * is generated. The output buffer must be allocated by the caller and have at
+ * least the length of the message digest size for the chosen message digest.
+ *
+ * @return size of message digest upon success;
+ *	    -EIO - data cannot be obtained;
+ * 	    -ENOMEM - buffer is too small for the complete message digest,
+ * 	    the buffer is filled with the truncated message digest
+ */
+int32_t kcapi_md_sha256(const uint8_t *in, uint32_t inlen,
+			uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_md_sha384 - SHA-384 message digest on one buffer
+ *
+ * @in: [in] buffer with input data
+ * @inlen: [in] length of input buffer
+ * @out: [out] buffer for message digest
+ * @outlen: [in] length of out
+ *
+ * With this one-shot convenience function, a message digest of the given buffer
+ * is generated. The output buffer must be allocated by the caller and have at
+ * least the length of the message digest size for the chosen message digest.
+ *
+ * @return size of message digest upon success;
+ *	    -EIO - data cannot be obtained;
+ * 	    -ENOMEM - buffer is too small for the complete message digest,
+ * 	    the buffer is filled with the truncated message digest
+ */
+int32_t kcapi_md_sha384(const uint8_t *in, uint32_t inlen,
+			uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_md_sha512 - SHA-512 message digest on one buffer
+ *
+ * @in: [in] buffer with input data
+ * @inlen: [in] length of input buffer
+ * @out: [out] buffer for message digest
+ * @outlen: [in] length of out
+ *
+ * With this one-shot convenience function, a message digest of the given buffer
+ * is generated. The output buffer must be allocated by the caller and have at
+ * least the length of the message digest size for the chosen message digest.
+ *
+ * @return size of message digest upon success;
+ *	    -EIO - data cannot be obtained;
+ * 	    -ENOMEM - buffer is too small for the complete message digest,
+ * 	    the buffer is filled with the truncated message digest
+ */
+int32_t kcapi_md_sha512(const uint8_t *in, uint32_t inlen,
+			uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_md_hmac_sha1 - HMAC SHA-1 keyed message digest on one buffer
+ *
+ * @key: [in] buffer with HMAC key
+ * @keylen: [in] length of HMAC key buffer
+ * @in: [in] buffer with input data
+ * @inlen: [in] length of input buffer
+ * @out: [out] buffer for message digest
+ * @outlen: [in] length of out
+ *
+ * With this one-shot convenience function, a keyed message digest of the given
+ * buffer is generated. The output buffer must be allocated by the caller and
+ * have at least the length of the message digest size for the chosen keyed
+ * message digest.
+ *
+ * @return size of message digest upon success;
+ *	    -EIO - data cannot be obtained;
+ * 	    -ENOMEM - buffer is too small for the complete message digest,
+ * 	    the buffer is filled with the truncated message digest
+ */
+int32_t kcapi_md_hmac_sha1(const uint8_t *key, uint32_t keylen,
+			   const uint8_t *in, uint32_t inlen,
+			   uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_md_hmac_sha224 - HMAC SHA-224 keyed message digest on one buffer
+ *
+ * @key: [in] buffer with HMAC key
+ * @keylen: [in] length of HMAC key buffer
+ * @in: [in] buffer with input data
+ * @inlen: [in] length of input buffer
+ * @out: [out] buffer for message digest
+ * @outlen: [in] length of out
+ *
+ * With this one-shot convenience function, a keyed message digest of the given
+ * buffer is generated. The output buffer must be allocated by the caller and
+ * have at least the length of the message digest size for the chosen keyed
+ * message digest.
+ *
+ * @return size of message digest upon success;
+ *	    -EIO - data cannot be obtained;
+ * 	    -ENOMEM - buffer is too small for the complete message digest,
+ * 	    the buffer is filled with the truncated message digest
+ */
+int32_t kcapi_md_hmac_sha224(const uint8_t *key, uint32_t keylen,
+			     const uint8_t *in, uint32_t inlen,
+			     uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_md_hmac_sha256 - HMAC SHA-256 keyed message digest on one buffer
+ *
+ * @key: [in] buffer with HMAC key
+ * @keylen: [in] length of HMAC key buffer
+ * @in: [in] buffer with input data
+ * @inlen: [in] length of input buffer
+ * @out: [out] buffer for message digest
+ * @outlen: [in] length of out
+ *
+ * With this one-shot convenience function, a keyed message digest of the given
+ * buffer is generated. The output buffer must be allocated by the caller and
+ * have at least the length of the message digest size for the chosen keyed
+ * message digest.
+ *
+ * @return size of message digest upon success;
+ *	    -EIO - data cannot be obtained;
+ * 	    -ENOMEM - buffer is too small for the complete message digest,
+ * 	    the buffer is filled with the truncated message digest
+ */
+int32_t kcapi_md_hmac_sha256(const uint8_t *key, uint32_t keylen,
+			     const uint8_t *in, uint32_t inlen,
+			     uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_md_hmac_sha384 - HMAC SHA-384 keyed message digest on one buffer
+ *
+ * @key: [in] buffer with HMAC key
+ * @keylen: [in] length of HMAC key buffer
+ * @in: [in] buffer with input data
+ * @inlen: [in] length of input buffer
+ * @out: [out] buffer for message digest
+ * @outlen: [in] length of out
+ *
+ * With this one-shot convenience function, a keyed message digest of the given
+ * buffer is generated. The output buffer must be allocated by the caller and
+ * have at least the length of the message digest size for the chosen keyed
+ * message digest.
+ *
+ * @return size of message digest upon success;
+ *	    -EIO - data cannot be obtained;
+ * 	    -ENOMEM - buffer is too small for the complete message digest,
+ * 	    the buffer is filled with the truncated message digest
+ */
+int32_t kcapi_md_hmac_sha384(const uint8_t *key, uint32_t keylen,
+			     const uint8_t *in, uint32_t inlen,
+			     uint8_t *out, uint32_t outlen);
+
+/**
+ * kcapi_md_hmac_sha512 - HMAC SHA-512 keyed message digest on one buffer
+ *
+ * @key: [in] buffer with HMAC key
+ * @keylen: [in] length of HMAC key buffer
+ * @in: [in] buffer with input data
+ * @inlen: [in] length of input buffer
+ * @out: [out] buffer for message digest
+ * @outlen: [in] length of out
+ *
+ * With this one-shot convenience function, a keyed message digest of the given
+ * buffer is generated. The output buffer must be allocated by the caller and
+ * have at least the length of the message digest size for the chosen keyed
+ * message digest.
+ *
+ * @return size of message digest upon success;
+ *	    -EIO - data cannot be obtained;
+ * 	    -ENOMEM - buffer is too small for the complete message digest,
+ * 	    the buffer is filled with the truncated message digest
+ */
+int32_t kcapi_md_hmac_sha512(const uint8_t *key, uint32_t keylen,
+			     const uint8_t *in, uint32_t inlen,
+			     uint8_t *out, uint32_t outlen);
+
 /**
  * kcapi_md_digestsize() - return the size of the message digest
  *
@@ -1216,6 +1542,20 @@ int kcapi_rng_seed(struct kcapi_handle *handle, uint8_t *seed,
 int32_t kcapi_rng_generate(struct kcapi_handle *handle,
 			   uint8_t *buffer, uint32_t len);
 
+/**
+ * kcapi_rng_get_bytes - Convenience function to generate random bytes
+ *
+ * @buffer: [out] filled with the random number
+ * @outlen: [in] buffer length
+ *
+ * This convenience function generates random bytes of the size of outlen
+ * and stores them into the provided buffer.
+ *
+ * @return size of random number generated upon success;
+ *	   -EIO - data cannot be obtained
+ */
+int32_t kcapi_rng_get_bytes(uint8_t *buffer, uint32_t outlen);
+
 /**
  * kcapi_rng_seedsize() - return required seed size of DRNG
  *
@@ -1233,11 +1573,11 @@ uint32_t kcapi_rng_seedsize(struct kcapi_handle *handle);
  */
 
 enum kcapi_verbosity {
-	LOG_NONE,
-	LOG_ERR,
-	LOG_WARN,
-	LOG_VERBOSE,
-	LOG_DEBUG,
+	KCAPI_LOG_NONE,
+	KCAPI_LOG_ERR,
+	KCAPI_LOG_WARN,
+	KCAPI_LOG_VERBOSE,
+	KCAPI_LOG_DEBUG,
 };
 
 /**
@@ -1372,7 +1712,8 @@ void kcapi_akcipher_destroy(struct kcapi_handle *handle);
  * After the caller provided the key, the caller may securely destroy the key
  * as it is now maintained by the kernel.
  *
- * @return 0 upon success;
+ * @return upon success the value of the maximum size for the asymmetric
+ *	   operation is returned (e.g. the modulus size);
  *	   a negative errno-style error code if an error occurred
  */
 int kcapi_akcipher_setkey(struct kcapi_handle *handle,
@@ -1397,7 +1738,8 @@ int kcapi_akcipher_setkey(struct kcapi_handle *handle,
  * After the caller provided the key, the caller may securely destroy the key
  * as it is now maintained by the kernel.
  *
- * @return 0 upon success;
+ * @return upon success the value of the maximum size for the asymmetric
+ *	   operation is returned (e.g. the modulus size);
  *	   a negative errno-style error code if an error occurred
  */
 int kcapi_akcipher_setpubkey(struct kcapi_handle *handle,
@@ -1767,6 +2109,11 @@ int32_t kcapi_akcipher_stream_init_vfy(struct kcapi_handle *handle,
  * where one thread sends data to be processed and one thread picks up data
  * processed by the cipher operation.
  *
+ * WARNING: The memory referenced by @iov is not accessed by the kernel
+ * during this call. The memory is first accessed when kcapi_cipher_stream_op()
+ * is called. Thus, you MUST make sure that the referenced memory is still
+ * present at the time kcapi_cipher_stream_op() is called.
+ *
  * @return number of bytes sent to the kernel upon success;
  *	   a negative errno-style error code if an error occurred
  */
@@ -1791,6 +2138,11 @@ int32_t kcapi_akcipher_stream_update(struct kcapi_handle *handle,
  * kcapi_akcipher_stream_op() will be invoked as a next step. This call
  * notifies the kernel that no further data is to be expected.
  *
+ * WARNING: The memory referenced by @iov is not accessed by the kernel
+ * during this call. The memory is first accessed when kcapi_cipher_stream_op()
+ * is called. Thus, you MUST make sure that the referenced memory is still
+ * present at the time kcapi_cipher_stream_op() is called.
+ *
  * @return number of bytes sent to the kernel upon success;
  *	   a negative errno-style error code if an error occurred
  */
@@ -1829,10 +2181,215 @@ int32_t kcapi_akcipher_stream_op(struct kcapi_handle *handle,
  * API function calls used to invoke Diffie-Hellmand or EC-Diffie-Hellman
  * operations.
  */
-/* Curves IDs */
+
+/**
+ * kcapi_kpp_init() - initialize cipher handle
+ *
+ * @handle: [out] cipher handle filled during the call
+ * @ciphername: [in] kernel crypto API cipher name as specified in
+ *	/proc/crypto
+ * @flags: [in] flags specifying the type of cipher handle
+ *
+ * This function provides the initialization of a KPP cipher handle and
+ * establishes the connection to the kernel.
+ *
+ * On success, a pointer to kcapi_handle object is returned in *handle.
+ * Function kcapi_kpp_destroy should be called afterwards to free
+ * resources.
+ *
+ * @return 0 upon success;
+ *	   -ENOENT - algorithm not available;
+ *	   -EOPNOTSUPP - AF_ALG family not available;
+ *	   -EINVAL - accept syscall failed
+ *	   -ENOMEM - cipher handle cannot be allocated
+ */
+int kcapi_kpp_init(struct kcapi_handle **handle, const char *ciphername,
+		   uint32_t flags);
+
+/**
+ * kcapi_kpp_destroy() - close the cipher handle and release resources
+ *
+ * @handle: [in] cipher handle to release
+ */
+void kcapi_kpp_destroy(struct kcapi_handle *handle);
+
+/**
+ * kcapi_kpp_dh_setparam_pkcs3 - set the PG parameters using PKCS3 format
+ *
+ * @handle: [in] cipher handle
+ * @pkcs3: [in] parameter buffer in DER format
+ * @pkcs3len: [in] length of key buffer
+ *
+ * With this function, the caller sets the PG parameters for subsequent cipher
+ * operations.
+ *
+ * The parameter set must be in DER format as follows
+ *
+ * SEQUENCE {
+ *	prime INTEGER ({ dh_get_p }),
+ *	base INTEGER ({ dh_get_g })
+ *}
+ *
+ * The following command generates such parameter set where the output
+ * file content is has the correct DER structure:
+ *
+ * openssl dhparam -outform DER -out dhparam.der 2048
+ *
+ * Note, this function defines that the subsequent key generation and
+ * shared secret operation performs an FFC Diffie-Hellman operation.
+ *
+ * After the caller provided the key, the caller may destroy the parameter
+ * as it is now maintained by the kernel.
+ *
+ * @return upon success the value of the maximum size for the KPP
+ *	   operation is returned (e.g. the prime size);
+ *	   a negative errno-style error code if an error occurred
+ */
+int kcapi_kpp_dh_setparam_pkcs3(struct kcapi_handle *handle,
+				const uint8_t *pkcs3, uint32_t pkcs3len);
+
+/* ECC curve IDs */
 #define ECC_CURVE_NIST_P192     0x0001
 #define ECC_CURVE_NIST_P256     0x0002
 
+/**
+ * kcapi_kpp_ecdh_setcurve - set the ECC curve to be used for ECDH
+ *
+ * @handle: [in] cipher handle
+ * @curve_id: [in] ID of the ECC curve
+ *
+ * With this function, the caller sets the ECC curve for subsequent cipher
+ * operations. The curve ID is one of the ECC_CURVE_* identifiers.
+ *
+ * Note, this function defines that the subsequent key generation and
+ * shared secret operation performs an ECC Diffie-Hellman operation.
+ *
+ * @return 0 upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int kcapi_kpp_ecdh_setcurve(struct kcapi_handle *handle,
+			    unsigned long curve_id);
+
+/**
+ * kcapi_kpp_setkey - set the private key of the DH / ECDH operation
+ *
+ * @handle: [in] cipher handle
+ * @key: [in] key buffer
+ * @keylen: [in] length of key buffer
+ *
+ * With this function, the caller sets the key for subsequent DH / ECDH
+ * public key generation or shared secret generation operations.
+ *
+ * If the key / keylen is zero, the kernel tries to generate the private key
+ * itself and retains it internally. This is useful if the DH / ECDH operation
+ * shall be performed on ephemeral keys where the caller is only interested
+ * in eventually obtain the shared secret.
+ *
+ * After the caller provided the key, the caller may securely destroy the key
+ * as it is now maintained by the kernel.
+ *
+ * Note, the key can only be set after the DH parameters or the ECC curve
+ * has been set.
+ *
+ * @return in case of success a positive integer is returned that denominates
+ *	   the maximum output size of the cryptographic operation -- this value
+ *	   must be used as the size of the output buffer for one cryptographic
+ *	   operation);
+ *	   a negative errno-style error code if an error occurred -- the error
+ *	   -EOPNOTSUPP is returned in case a kernel-triggered private
+ *	   key generation is requested, but the underlying cipher implementation
+ *	   does not support this operation.
+ */
+int kcapi_kpp_setkey(struct kcapi_handle *handle,
+		     const uint8_t *key, uint32_t keylen);
+
+/**
+ * kcapi_kpp_keygen - generate a public key
+ *
+ * @handle: [in] cipher handle
+ * @pubkey: [out] generated public key
+ * @pubkeylen: [in] length of key buffer
+ * @access: [in] kernel access type (KCAPI_ACCESS_HEURISTIC - use internal
+ *	    heuristic for  fastest kernel access; KCAPI_ACCESS_VMSPLICE - use
+ *	    vmsplice access; KCAPI_ACCESS_SENDMSG - sendmsg access)
+ *
+ * @return number of bytes returned by the key generation operation upon
+ *	   success; a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_kpp_keygen(struct kcapi_handle *handle,
+			 uint8_t *pubkey, uint32_t pubkeylen, int access);
+
+/**
+ * kcapi_kpp_ssgen - generate a shared secret
+ *
+ * @handle: [in] cipher handle
+ * @pubkey: [in] public key of peer that shall be used to generate the shared
+ *	    secret with
+ * @pubkeylen: [in] length of the public key buffer
+ * @ss: [out] generated shared secret
+ * @sslen: [in] length of key buffer
+ * @access: [in] kernel access type (KCAPI_ACCESS_HEURISTIC - use internal
+ *	    heuristic for  fastest kernel access; KCAPI_ACCESS_VMSPLICE - use
+ *	    vmsplice access; KCAPI_ACCESS_SENDMSG - sendmsg access)
+ *
+ * @return number of bytes returned by the shared secret generation operation
+ *	   upon success; a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_kpp_ssgen(struct kcapi_handle *handle,
+			const uint8_t *pubkey, uint32_t pubkeylen,
+			uint8_t *ss, uint32_t sslen, int access);
+
+/**
+ * kcapi_kpp_keygen_aio() - generate a public key (asynchronous one shot)
+ *
+ * @handle: [in] cipher handle
+ * @outiov: [out] head of scatter-gather list of the destination buffers filled
+ *	with the generated public key
+ * @iovlen: [in] number of scatter-gather list entries
+ * @access: [in] kernel access type (KCAPI_ACCESS_HEURISTIC - use internal
+ *	heuristic for  fastest kernel access; KCAPI_ACCESS_VMSPLICE - use
+ *	vmsplice access; KCAPI_ACCESS_SENDMSG - sendmsg access)
+ *
+ * The individual scatter-gather list entries are processed with
+ * separate invocations of the the given cipher.
+ *
+ * The memory should be aligned at the page boundary using
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
+ *
+ * @return number of bytes verify upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_kpp_keygen_aio(struct kcapi_handle *handle, struct iovec *outiov,
+			     uint32_t iovlen, int access);
+
+/**
+ * kcapi_kpp_ssgen_aio() - generate a shared secret (asynchronous one shot)
+ *
+ * @handle: [in] cipher handle
+ * @iniov: [in] head of scatter-gather list of the source buffers with the
+ *	public keys of the peer
+ * @outiov: [out] head of scatter-gather list of the destination buffers filled
+ *	with the generated shared secret
+ * @iovlen: [in] number of scatter-gather list entries
+ * @access: [in] kernel access type (KCAPI_ACCESS_HEURISTIC - use internal
+ *	heuristic for  fastest kernel access; KCAPI_ACCESS_VMSPLICE - use
+ *	vmsplice access; KCAPI_ACCESS_SENDMSG - sendmsg access)
+ *
+ * The individual scatter-gather list entries are processed with
+ * separate invocations of the the given cipher.
+ *
+ * The memory should be aligned at the page boundary using
+ * posix_memalign(sysconf(_SC_PAGESIZE)), If it is not aligned at the page
+ * boundary, the vmsplice call may not send all data to the kernel.
+ *
+ * @return number of bytes verify upon success;
+ *	   a negative errno-style error code if an error occurred
+ */
+int32_t kcapi_kpp_ssgen_aio(struct kcapi_handle *handle,
+			    struct iovec *iniov, struct iovec *outiov,
+			    uint32_t iovlen, int access);
+
 /**
  * DOC: Key Derivation Functions
  *
